/*
 * STRUMPACK -- STRUctured Matrices PACKage, Copyright (c) 2014, The Regents of
 * the University of California, through Lawrence Berkeley National Laboratory
 * (subject to receipt of any required approvals from the U.S. Dept. of Energy).
 * All rights reserved.
 *
 * If you have questions about your rights to use or distribute this software,
 * please contact Berkeley Lab's Technology Transfer Department at TTD@lbl.gov.
 *
 * NOTICE. This software is owned by the U.S. Department of Energy. As such, the
 * U.S. Government has been granted for itself and others acting on its behalf a
 * paid-up, nonexclusive, irrevocable, worldwide license in the Software to
 * reproduce, prepare derivative works, and perform publicly and display publicly.
 * Beginning five (5) years after the date permission to assert copyright is
 * obtained from the U.S. Department of Energy, and subject to any subsequent five
 * (5) year renewals, the U.S. Government is granted for itself and others acting
 * on its behalf a paid-up, nonexclusive, irrevocable, worldwide license in the
 * Software to reproduce, prepare derivative works, distribute copies to the
 * public, perform publicly and display publicly, and to permit others to do so.
 *
 * Developers: Francois-Henry Rouet, Xiaoye S. Li, Pieter Ghysels
 *             (Lawrence Berkeley National Lab, Computational Research Division).
 *
 */

#ifndef LAPACK_HPP
#define LAPACK_HPP

/* LAPACK routines */

  /* xLACPY */
  template<typename T> inline int lacpy(char , int , int , T *, int , T *, int);
  template<> inline int lacpy<  double>(char uplo, int m, int n,   double *A, int lda,   double *B, int ldb) {
    return dlacpy_(&uplo,&m,&n,A,&lda,B,&ldb);
  }
  template<> inline int lacpy<   float>(char uplo, int m, int n,    float *A, int lda,    float *B, int ldb) {
    return slacpy_(&uplo,&m,&n,A,&lda,B,&ldb);
  }
  template<> inline int lacpy<dcomplex>(char uplo, int m, int n, dcomplex *A, int lda, dcomplex *B, int ldb) {
    return zlacpy_(&uplo,&m,&n,A,&lda,B,&ldb);
  }
  template<> inline int lacpy<scomplex>(char uplo, int m, int n, scomplex *A, int lda, scomplex *B, int ldb) {
    return clacpy_(&uplo,&m,&n,A,&lda,B,&ldb);
  }

  /* xLAPMT */
  template<typename T> inline void lapmt(int, int, int, T *, int, int *);
  template<> inline void lapmt<  double>(int fwd, int m, int n,   double *A, int lda, int *P) {
    dlapmt_(&fwd,&m,&n,A,&lda,P);
  }
  template<> inline void lapmt<   float>(int fwd, int m, int n,    float *A, int lda, int *P) {
    slapmt_(&fwd,&m,&n,A,&lda,P);
  }
  template<> inline void lapmt<dcomplex>(int fwd, int m, int n, dcomplex *A, int lda, int *P) {
    zlapmt_(&fwd,&m,&n,A,&lda,P);
  }
  template<> inline void lapmt<scomplex>(int fwd, int m, int n, scomplex *A, int lda, int *P) {
    clapmt_(&fwd,&m,&n,A,&lda,P);
  }

  /* xLANGE */
  template<typename T, typename S> inline S lange(char , int , int , T *, int , S *);
  template<> inline double lange<  double,double>(char norm, int m, int n,   double *A, int lda, double *work) {
    return dlange_(&norm,&m,&n,A,&lda,work);
  }
  template<> inline  float lange<   float, float>(char norm, int m, int n,    float *A, int lda,  float *work) {
    return slange_(&norm,&m,&n,A,&lda,work);
  }
  template<> inline double lange<dcomplex,double>(char norm, int m, int n, dcomplex *A, int lda, double *work) {
    return zlange_(&norm,&m,&n,A,&lda,work);
  }
  template<> inline  float lange<scomplex, float>(char norm, int m, int n, scomplex *A, int lda,  float *work) {
    return clange_(&norm,&m,&n,A,&lda,work);
}

  /* xLACGV */
  template<typename T> inline void lacgv(int, T *, int);
  template<> inline void lacgv<  double>(int n,   double *X, int incx) {
    //Nothing to do.
  }
  template<> inline void lacgv<   float>(int n,    float *X, int incx) {
    //Nothing to do.
  }
  template<> inline void lacgv<dcomplex>(int n, dcomplex *X, int incx) {
    zlacgv_(&n,X,&incx);
  }
  template<> inline void lacgv<scomplex>(int n, scomplex *X, int incx) {
    clacgv_(&n,X,&incx);
  }

  /* xGETRF */
  template<typename T> inline int getrf(int, int, T *, int, int *);
  template<> inline int getrf<  double>(int m, int n,   double *A, int lda, int *ipiv) {
    int info;
    dgetrf_(&m,&n,A,&lda,ipiv,&info);
    return info;
  }
  template<> inline int getrf<   float>(int m, int n,    float *A, int lda, int *ipiv) {
    int info;
    sgetrf_(&m,&n,A,&lda,ipiv,&info);
    return info;
  }
  template<> inline int getrf<dcomplex>(int m, int n, dcomplex *A, int lda, int *ipiv) {
    int info;
    zgetrf_(&m,&n,A,&lda,ipiv,&info);
    return info;
  }
  template<> inline int getrf<scomplex>(int m, int n, scomplex *A, int lda, int *ipiv) {
    int info;
    cgetrf_(&m,&n,A,&lda,ipiv,&info);
    return info;
  }

  /* xGETRS */
  template<typename T> inline int getrs(char, int, int, T *, int, int *, T *, int);
  template<> inline int getrs<  double>(char uplo, int n, int nrhs,   double *A, int lda, int *ipiv,   double *B, int ldb) {
    int info;
    dgetrs_(&uplo,&n,&nrhs,A,&lda,ipiv,B,&ldb,&info);
    return info;
  }
  template<> inline int getrs<   float>(char uplo, int n, int nrhs,    float *A, int lda, int *ipiv,    float *B, int ldb) {
    int info;
    sgetrs_(&uplo,&n,&nrhs,A,&lda,ipiv,B,&ldb,&info);
    return info;
  }
  template<> inline int getrs<dcomplex>(char uplo, int n, int nrhs, dcomplex *A, int lda, int *ipiv, dcomplex *B, int ldb) {
    int info;
    zgetrs_(&uplo,&n,&nrhs,A,&lda,ipiv,B,&ldb,&info);
    return info;
  }
  template<> inline int getrs<scomplex>(char uplo, int n, int nrhs, scomplex *A, int lda, int *ipiv, scomplex *B, int ldb) {
    int info;
    cgetrs_(&uplo,&n,&nrhs,A,&lda,ipiv,B,&ldb,&info);
    return info;
  }

  /* xGELQF */
  template<typename T> inline void gelqf(int, int, T *, int, T *);
  template<> inline void gelqf(int m, int n, double *A, int lda, double *tau) {
    double *work, dlwork;
    int lwork, info;

    /* Query call */
    dgelqf_(&m,&n,A,&lda,tau,&dlwork,&INONE,&info);
    lwork=static_cast<int>(dlwork);
    work=new double[lwork];

    /* Actual call */
    dgelqf_(&m,&n,A,&lda,tau,work,&lwork,&info);

    delete []work;
  }
  template<> inline void gelqf(int m, int n, float *A, int lda, float *tau) {
    float *work, dlwork;
    int lwork, info;

    /* Query call */
    sgelqf_(&m,&n,A,&lda,tau,&dlwork,&INONE,&info);
    lwork=static_cast<int>(dlwork);
    work=new float[lwork];

    /* Actual call */
    sgelqf_(&m,&n,A,&lda,tau,work,&lwork,&info);

    delete []work;
  }
  template<> inline void gelqf(int m, int n, dcomplex *A, int lda, dcomplex *tau) {
    dcomplex *work, dlwork;
    int lwork, info;

    /* Query call */
    zgelqf_(&m,&n,A,&lda,tau,&dlwork,&INONE,&info);
    lwork=static_cast<int>(dlwork.real());
    work=new dcomplex[lwork];

    /* Actual call */
    zgelqf_(&m,&n,A,&lda,tau,work,&lwork,&info);

    delete []work;
  }
  template<> inline void gelqf(int m, int n, scomplex *A, int lda, scomplex *tau) {
    scomplex *work, dlwork;
    int lwork, info;

    /* Query call */
    cgelqf_(&m,&n,A,&lda,tau,&dlwork,&INONE,&info);
    lwork=static_cast<int>(dlwork.real());
    work=new scomplex[lwork];

    /* Actual call */
    cgelqf_(&m,&n,A,&lda,tau,work,&lwork,&info);

    delete []work;
  }

  /* xxxGLQ */
  template<typename T> inline void xxglq(int, int, int, T *, int, T *);
  template<> inline void xxglq(int m, int n, int k, double *A, int lda, double *tau) {
    double *work, dlwork;
    int lwork, info;

    /* Query call */
    dorglq_(&m,&n,&k,A,&lda,tau,&dlwork,&INONE,&info);
    lwork=static_cast<int>(dlwork);
    work=new double[lwork];

    /* Actual call */
    dorglq_(&m,&n,&k,A,&lda,tau,work,&lwork,&info);

    delete []work;
  }
  template<> inline void xxglq(int m, int n, int k, float *A, int lda, float *tau) {
    float *work, dlwork;
    int lwork, info;

    /* Query call */
    sorglq_(&m,&n,&k,A,&lda,tau,&dlwork,&INONE,&info);
    lwork=static_cast<int>(dlwork);
    work=new float[lwork];

    /* Actual call */
    sorglq_(&m,&n,&k,A,&lda,tau,work,&lwork,&info);

    delete []work;
  }
  template<> inline void xxglq(int m, int n, int k, dcomplex *A, int lda, dcomplex *tau) {
    dcomplex *work, dlwork;
    int lwork, info;

    /* Query call */
    zunglq_(&m,&n,&k,A,&lda,tau,&dlwork,&INONE,&info);
    lwork=static_cast<int>(dlwork.real());
    work=new dcomplex[lwork];

    /* Actual call */
    zunglq_(&m,&n,&k,A,&lda,tau,work,&lwork,&info);

    delete []work;
  }
  template<> inline void xxglq(int m, int n, int k, scomplex *A, int lda, scomplex *tau) {
    scomplex *work, dlwork;
    int lwork, info;

    /* Query call */
    cunglq_(&m,&n,&k,A,&lda,tau,&dlwork,&INONE,&info);
    lwork=static_cast<int>(dlwork.real());
    work=new scomplex[lwork];

    /* Actual call */
    cunglq_(&m,&n,&k,A,&lda,tau,work,&lwork,&info);

    delete []work;
  }

  /* xGEQP3mod */
  template<typename T, typename S> inline void geqp3mod(int, int, T *, int *, int *, S);
  template<> inline void geqp3mod(int m, int n, double *A, int * J, int *r, double tol) {
    double *tau, *work, dlwork;
    int lwork, info;

    tau=new double[std::max(1,std::min(m,n))];

    /* Query call */
    dgeqp3mod_(&m,&n,A,&m,J,tau,&dlwork,&INONE,&info,r,&tol);
    lwork=static_cast<int>(dlwork);
    work=new double[lwork];

    /* Actual call */
    dgeqp3mod_(&m,&n,A,&m,J,tau,work,&lwork,&info,r,&tol);

    delete []tau;
    delete []work;
  }
  template<> inline void geqp3mod(int m, int n, float *A, int * J, int *r, float tol) {
    float *tau, *work, dlwork;
    int lwork, info;

    tau=new float[n];

    /* Query call */
    sgeqp3mod_(&m,&n,A,&m,J,tau,&dlwork,&INONE,&info,r,&tol);
    lwork=static_cast<int>(dlwork);
    work=new float[lwork];

    /* Actual call */
    sgeqp3mod_(&m,&n,A,&m,J,tau,work,&lwork,&info,r,&tol);

    delete []tau;
    delete []work;
  }
  template<> inline void geqp3mod(int m, int n, dcomplex *A, int * J, int *r, double tol) {
    dcomplex *tau, *work, dlwork;
    double *rwork;
    int lwork, info;

    tau=new dcomplex[n];
    rwork=new double[2*n];

    /* Query call */
    zgeqp3mod_(&m,&n,A,&m,J,tau,&dlwork,&INONE,rwork,&info,r,&tol);
    lwork=static_cast<int>(dlwork.real());
    work=new dcomplex[lwork];

    /* Actual call */
    zgeqp3mod_(&m,&n,A,&m,J,tau,work,&lwork,rwork,&info,r,&tol);

    delete []tau;
    delete []work;
    delete []rwork;
  }
  template<> inline void geqp3mod(int m, int n, scomplex *A, int * J, int *r,  float tol) {
    scomplex *tau, *work, dlwork;
    float *rwork;
    int lwork, info;

    tau=new scomplex[n];
    rwork=new float[2*n];

    /* Query call */
    cgeqp3mod_(&m,&n,A,&m,J,tau,&dlwork,&INONE,rwork,&info,r,&tol);
    lwork=static_cast<int>(dlwork.real());
    work=new scomplex[lwork];

    /* Actual call */
    cgeqp3mod_(&m,&n,A,&m,J,tau,work,&lwork,rwork,&info,r,&tol);

    delete []tau;
    delete []work;
    delete []rwork;
  }

  /* xGEQP4mod */
  template<typename T, typename S> inline void geqp4mod(int, int, T *, int *, int *, S);
  template<> inline void geqp4mod(int m, int n, double *A, int * J, int *r, double tol) {
    double *tau, *work, dlwork;
    int lwork, info;

    tau=new double[std::max(1,std::min(m,n))];

    /* Query call */
    dgeqp4mod(&m,&n,A,&m,J,tau,&dlwork,&INONE,&info,r,&tol);
    lwork=static_cast<int>(dlwork);
    work=new double[lwork];

    /* Actual call */
    dgeqp4mod(&m,&n,A,&m,J,tau,work,&lwork,&info,r,&tol);

    delete []tau;
    delete []work;
  }
  template<> inline void geqp4mod(int m, int n, float *A, int * J, int *r, float tol) {
  }
  template<> inline void geqp4mod(int m, int n, dcomplex *A, int * J, int *r, double tol) {
  }
  template<> inline void geqp4mod(int m, int n, scomplex *A, int * J, int *r,  float tol) {
  }

  /* xLASWP */
  template<typename T> inline void laswp(int, T *, int, int, int, int *, int);
  template<> inline void laswp<  double>(int n,   double *A, int lda, int k1, int k2, int *ipiv, int incx) {
    dlaswp_(&n,A,&lda,&k1,&k2,ipiv,&incx);
  }
  template<> inline void laswp<   float>(int n,    float *A, int lda, int k1, int k2, int *ipiv, int incx) {
    slaswp_(&n,A,&lda,&k1,&k2,ipiv,&incx);
  }
  template<> inline void laswp<dcomplex>(int n, dcomplex *A, int lda, int k1, int k2, int *ipiv, int incx) {
    zlaswp_(&n,A,&lda,&k1,&k2,ipiv,&incx);
  }
  template<> inline void laswp<scomplex>(int n, scomplex *A, int lda, int k1, int k2, int *ipiv, int incx) {
    claswp_(&n,A,&lda,&k1,&k2,ipiv,&incx);
  }

#endif // LAPACK_HPP
